#!/usr/bin/perl -w   #output all warnings 

#####################################################################################
#   HybridNumber.pl                                                                
#                                                                           
#   Algorithm to calculate the exact number of hybridization events for 
#   two given rooted binary phylogenetic trees on the same set of taxa                                                   
#                                                                            
#   Copyright (C) 2006 by Magnus Bordewich, Simone Linz, Katherine St. John,
#   and Charles Semple
#
#   This is free software and may be distributed under the same terms as Perl itself.
#        
#   linz@cs.uni-duesseldorf.de
#####################################################################################

use lib ".";   


require tree;
require subtree;
require chain;
require cluster;
require forest;

$| = 1;   #no output buffering

if (@ARGV!=2){
	print "usage perl count_hybrid.pl infile1 infile2 > output_dir/logfile\n\n";
	die;
}

my $file1=$ARGV[0]; 
my $file2=$ARGV[1];  


#read files and sort trees
my @tree1=tree::read_tree_string($file1);
my @tree2=tree::read_tree_string($file2);

######################################################
#test if arrays are the same
if(tree::compare_arrays(\@tree1, \@tree2)){
	print "trees are the same!\n";
	exit;
}

#test if the leaf set of both trees is the same
if ($tree2[0] ne $tree1[0]){
	print "ERROR trees have not the same leave set\n\n";
	exit;
}

#output before starting the calculation
tree::print_array("tree1 input", \@tree1);
tree::print_array("tree2 input", \@tree2);

#subtree reduction rule
#-----------------------
my @set_p;  #chains with corresponding weights
(my $tree1_ref, my $tree2_ref, my $set_p_ref)=subtree::find_pendant_subtrees(\@tree1, \@tree2, \@set_p);

#output after the subtree reduction rule
tree::print_array("tree1 after subtree reduction", $tree1_ref);
tree::print_array("tree2 after subtree reduction", $tree2_ref);


#chain reduction rule
#---------------------
($tree1_ref, $tree2_ref, $set_p_ref)=chain::find_maximal_chains($tree1_ref, $tree2_ref, $set_p_ref);

#test output
tree::print_array("tree1 after chain reduction", $tree1_ref);
tree::print_array("tree2 after chain reduction", $tree2_ref);

#cluster reduction and forest calculation
#----------------------------------------

my @cluster1;
my @cluster2;
my @current_set_p;
my $current_set_p_ref;
my $cluster1_ref;
my $cluster2_ref;

my @forest;
my $hybridization_number=0;


while (@$tree1_ref>1){   #stop if trees only consist of a single vertex (no more reductions are possible)
  ($cluster1_ref, $cluster2_ref, $tree1_ref, $tree2_ref)=cluster::find_cluster($tree1_ref, $tree2_ref);
  tree::print_array("cluster 1 ", $cluster1_ref);
  tree::print_array("cluster 2 ", $cluster2_ref);
  ($set_p_ref, $current_set_p_ref)=cluster::update_set_p($set_p_ref, $cluster1_ref);
  #FIND FOREST OF $CURRENT_CLUSTER1_REF AND $CURRENT_CLUSTER2_REF AND UPDATE $HYBRIDIZATION_NUMBER
  #print "search for forest of current cluster\n";
  my $curr_hybrid=forest::find_forest($cluster1_ref, $cluster2_ref, $current_set_p_ref);
  print "hybridization number of current cluster is $curr_hybrid\n\n";
  $hybridization_number=$hybridization_number+$curr_hybrid;
  #T and T' could be the same after cutting a cluster
  if(tree::compare_arrays($tree1_ref, $tree2_ref)){  
    @tree1=@$tree1_ref;
    @tree2=@$tree2_ref;
    my $dummy=$tree1[0];
    @tree1="";
    $tree1[0]=$dummy;
    $dummy=$tree2[0];
    @tree2="";
    $tree2[0]=$dummy;
    $tree1_ref=\@tree1;
    $tree2_ref=\@tree2;
  }
  else{		#apply the reduction rules again
    ($tree1_ref, $tree2_ref, $set_p_ref)=subtree::find_pendant_subtrees($tree1_ref, $tree2_ref, $set_p_ref);
    ($tree1_ref, $tree2_ref, $set_p_ref)=chain::find_maximal_chains($tree1_ref, $tree2_ref, $set_p_ref);
  }  
}


if ((@$tree1_ref>1)||(@$tree2_ref>1)){
  print "ERROR cluster reduction: only one tree consists of a single vertex in the end\n\n";
  die;
}

print "\nTHE TOTAL MINIMUM HYBRIDIZATION NUMBER IS $hybridization_number\n\n\n";
